from database import db, app
from flask_covid19.blueprints.app_all.all_config import BlueprintConfig
from flask_covid19.blueprints.data_rki.rki_model import RkiData, RkiMeldedatum, RkiBundesland, RkiLandkreis
from flask_covid19.blueprints.data_rki.rki_model_import import RkiImport


class RkiTestService:
    def __init__(self, database, rki_service):
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" RKI Test Service [init]")
        app.logger.debug("------------------------------------------------------------")
        self.__database = database
        self.__rki_service = rki_service
        self.cfg = BlueprintConfig.create_config_for_owid()
        app.logger.debug("------------------------------------------------------------")
        app.logger.info(" RKI Test Service [ready]")

    def delete_last_day(self):
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" RkiTestService.delete_last_day() [START]")
        app.logger.debug("------------------------------------------------------------")
        joungest_datum_str = RkiData.get_joungest_datum()
        joungest_datum = RkiMeldedatum.find_by_date_reported(joungest_datum_str)
        app.logger.info("joungest_datum:")
        app.logger.info(joungest_datum)
        app.logger.info("RkiData.get_data_for_one_day(joungest_datum):")
        i = 0
        for data in RkiData.get_data_for_one_day(joungest_datum):
            i += 1
            line = "Owid: to be deleted | " + str(i) + " | " + str(data.date_reported) + " | " + str(data.country) + " | "
            app.logger.info(line)
        app.logger.info("RkiData.delete_data_for_one_day(joungest_datum)")
        RkiData.delete_data_for_one_day(joungest_datum)
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" RkiTestService.delete_last_day() [DONE]")
        app.logger.debug("------------------------------------------------------------")

    def delete_last_continent(self):
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" RkiTestService.delete_last_continent() [START]")
        app.logger.debug("------------------------------------------------------------")
        app.logger.info("OwidContinent.get_last_continent()")
        last_continent = RkiBundesland.get_last_continent()
        app.logger.info("last_continent:" + str(last_continent))
        app.logger.info("OwidCountry.get_all_countries_for_continent(last_continent)")
        countries_for_continent = RkiLandkreis.get_all_countries_for_continent(last_continent)
        i = 0
        for country in countries_for_continent:
            for data in RkiData.get_all_data_for_country(country):
                i += 1
                line = "Owid: to be deleted | " + str(i) + " | " + str(data.date_reported) + " | " + str(data.country) + " | "
                app.logger.info(line)
            app.logger.info("RkiData.delete_all_data_for_country(country)")
            RkiData.delete_all_data_for_country(country)
        app.logger.info("RkiLandkreis.delete_all_countries_for_continent(last_continent)")
        RkiLandkreis.delete_all_countries_for_continent(last_continent)
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" RkiTestService.delete_last_continent() [DONE]")
        app.logger.debug("------------------------------------------------------------")

    def run_update_star_schema_incremental(self):
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" RkiTestService.run_update_star_schema_incremental() [START]")
        app.logger.debug("------------------------------------------------------------")
        self.__rki_service.update_star_schema()
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" RkiTestService.run_update_star_schema_incremental() [DONE]")
        app.logger.debug("------------------------------------------------------------")

    def full_update_dimension_tables(self):
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" RkiTestService.full_update_dimension_tables() [START]")
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug("")
        RkiData.remove_all()
        RkiLandkreis.remove_all()
        for continent in RkiBundesland.get_all():
            app.logger.info("continent.region: " + continent.region)
            for oi in RkiImport.get_countries_for_continent(continent.region):
                app.logger.info("continent.region: " + continent.region +" - oi.location: " + oi.location)
                o = RkiLandkreis(
                    continent_id=continent.id,
                    continent=continent,
                    location=oi.location,
                    iso_code=oi.iso_code,
                    population=oi.population,
                    population_density=oi.population_density,
                    median_age=oi.median_age,
                    aged_65_older=oi.aged_65_older,
                    aged_70_older=oi.aged_70_older,
                    gdp_per_capita=oi.gdp_per_capita,
                    extreme_poverty=oi.extreme_poverty,
                    cardiovasc_death_rate=oi.cardiovasc_death_rate,
                    diabetes_prevalence=oi.diabetes_prevalence,
                    female_smokers=oi.female_smokers,
                    male_smokers=oi.male_smokers,
                    handwashing_facilities=oi.handwashing_facilities,
                    hospital_beds_per_thousand=oi.hospital_beds_per_thousand,
                    life_expectancy=oi.life_expectancy,
                    human_development_index=oi.human_development_index
                )
                db.session.add(o)
        db.session.commit()
        app.logger.debug("")
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" RkiTestService.full_update_dimension_tables() [DONE]")
        app.logger.debug("------------------------------------------------------------")
